////////////////////////////////////////////////////////////////////////
// Class:       HitSubstract
// Module Type: producer
// File:        HitSubstract_module.cc
//
// Generated at Mon May 30 08:42:27 2016 by Andrey Sheshukov using artmod
// from cetpkgsupport v1_08_07.
////////////////////////////////////////////////////////////////////////

#include "art/Framework/Core/EDProducer.h"
#include "art/Framework/Core/ModuleMacros.h"
#include "art/Framework/Principal/Event.h"
#include "art/Framework/Principal/Handle.h"
#include "art/Framework/Principal/Run.h"
#include "art/Framework/Principal/SubRun.h"
#include "art/Utilities/InputTag.h"
#include "fhiclcpp/ParameterSet.h"
#include "messagefacility/MessageLogger/MessageLogger.h"
#include "DDTBaseDataProducts/HitList.h"
#include "DDTBaseDataProducts/HitSet.h"
#include <boost/algorithm/string.hpp>
#include <boost/tokenizer.hpp>
#include <exception>
#include <memory>
#include <string>
#include <vector>

/// a class for expression parsing. 
//Example: expr="A+B-C+D-E" means that we take hits from A
class Expression {
    public:
        Expression(const std::string& expr){Parse(expr);}
        novaddt::HitSet Eval(const art::Event & e);
    private:
        void Parse(const std::string& s);
        novaddt::HitSet fetchOne(const std::string& tag,const art::Event & e){
           auto handle=e.getValidHandle<novaddt::HitList>(tag);
           return novaddt::HitSet(*handle);
        };
        novaddt::HitSet fetchAll(const std::string& tag,const art::Event & e){
            novaddt::HitSet hits;
            auto handle=e.getValidHandle<std::vector<novaddt::HitList>>(tag);
            for(const auto& hitlist: *handle)
                hits.add(hitlist);
            return hits;
        };
        novaddt::HitSet fetch(const std::string& tag,const art::Event & e){
            if(boost::ends_with(tag,"[]")){
                return fetchAll(boost::erase_tail_copy(tag,2),e);
            }
            else
                return fetchOne(tag,e);
        }
    private:
        std::vector<std::string> fTokens;
};

void Expression::Parse(const std::string& str){
    mf::LogInfo("HitSub expression")<<"parse string:\'"<<str<<"\'";

    boost::char_separator<char> sep{"", "-+"};
    boost::tokenizer<boost::char_separator<char>> tok{str, sep};
    for (std::string t : tok){
        boost::trim(t);
       fTokens.push_back(t);
    }
}

novaddt::HitSet Expression::Eval(const art::Event & e){
   novaddt::HitSet result;
   char next_op='+';
   auto stream=mf::LogDebug("HitSub eval");
   for(const auto& t: fTokens){
      if(t[0]=='-'||t[0]=='+'){next_op=t[0]; continue;}
      novaddt::HitSet hits=fetch(t,e);
      stream<<next_op<<" Hits["<<t<<"]= "<<hits.size()<<"\n";
      switch(next_op){
        case '-': result.substract(hits); break;
        case '+': result.add(hits); break;
        default:  throw std::logic_error("Wrong operation:"+std::to_string(next_op));
      }
   }
   stream<<"Result="<<result.size();
   return result;
}

namespace novaddt {
  class HitSubstract;
}

class novaddt::HitSubstract : public art::EDProducer {
public:
  explicit HitSubstract(fhicl::ParameterSet const & p);
  // The destructor generated by the compiler is fine for classes
  // without bare pointers or other resource use.

  // Plugins should not be copied or assigned.
  HitSubstract(HitSubstract const &) = delete;
  HitSubstract(HitSubstract &&) = delete;
  HitSubstract & operator = (HitSubstract const &) = delete;
  HitSubstract & operator = (HitSubstract &&) = delete;

  // Required functions.
  void produce(art::Event & e) override;
    

private:
    Expression fExpr;
    std::string fLabel;
};


novaddt::HitSubstract::HitSubstract(fhicl::ParameterSet const & p)
 :fExpr(p.get<std::string>("expr")),fLabel(p.get<std::string>("label"))
// Initialize member data here.
{
  // Call appropriate produces<>() functions here.
  produces<novaddt::HitList>(fLabel);
}

void novaddt::HitSubstract::produce(art::Event & e)
{
  // Implementation of required member function here.
  auto hitSet=fExpr.Eval(e);
  std::unique_ptr<novaddt::HitList>hits(new novaddt::HitList(hitSet.begin(),hitSet.end()));
  e.put(std::move(hits),fLabel);
}

DEFINE_ART_MODULE(novaddt::HitSubstract)
