////////////////////////////////////////////////////////////////////////
// Class:       ChannelMapMaker
// Module Type: analyzer
// File:        ChannelMapMaker_module.cc
//
// Generated at Fri Feb 24 17:38:07 2017 by Andrey Sheshukov using artmod
// from cetpkgsupport v1_10_02.
////////////////////////////////////////////////////////////////////////

#include "art/Framework/Core/EDAnalyzer.h"
#include "art/Framework/Core/ModuleMacros.h"
#include "art/Framework/Principal/Event.h"
#include "art/Framework/Principal/Handle.h"
#include "art/Framework/Principal/Run.h"
#include "art/Framework/Principal/SubRun.h"
#include "art/Utilities/InputTag.h"
#include "fhiclcpp/ParameterSet.h"
#include "messagefacility/MessageLogger/MessageLogger.h"
#include "OnlineCalibration/HitMap.h"
#include "DDTBaseDataProducts/HitList.h"
#include <sstream>

namespace novaddt {
  class ChannelMapMaker;
}

class novaddt::ChannelMapMaker : public art::EDAnalyzer {
public:
  explicit ChannelMapMaker(fhicl::ParameterSet const & p);
  // The destructor generated by the compiler is fine for classes
  // without bare pointers or other resource use.

  // Plugins should not be copied or assigned.
  ChannelMapMaker(ChannelMapMaker const &) = delete;
  ChannelMapMaker(ChannelMapMaker &&) = delete;
  ChannelMapMaker & operator = (ChannelMapMaker const &) = delete;
  ChannelMapMaker & operator = (ChannelMapMaker &&) = delete;

  // Required functions.
  void analyze(art::Event const & e) override;
  void beginRun(art::Run const & r) override {
    //zero the map befor filling 
    fHitMap.reset(); 
    //zero the events counter
    fEventsProcessed=0;
  }
  void endRun(art::Run const & r) override{
    //normalize the map to 1 event
    fHitMap.Scale(1./fEventsProcessed);
    //save map to the file
    auto fname=prepareFileName(r);
    std::ofstream s(fname.data(),std::ofstream::binary);
    s<<fHitMap;
  }

  std::string prepareFileName(const art::Run& r){
    std::stringstream s;
    s<<fFileName<<".r"<<r.run()<<".nhm";
    return s.str();
  }
private:

  // Declare member data here.
    novaddt::calib::FDHitMap fHitMap;
    art::InputTag fHitsTag;
    std::string   fFileName;
    uint32_t fEventsProcessed=0;
};


novaddt::ChannelMapMaker::ChannelMapMaker(fhicl::ParameterSet const & p)
  :
  EDAnalyzer(p),
  fHitsTag(p.get<std::string>("hits_tag")),
  fFileName(p.get<std::string>("output_name"))
{}

void novaddt::ChannelMapMaker::analyze(art::Event const & e)
{
    //get the hits which we'll use to fill the map
    auto hits=e.getValidHandle<novaddt::HitList>(fHitsTag);
    //fill the map
    for(const auto &hit: *hits)
        fHitMap[hit]+=1;
    //Increment events counter
    ++fEventsProcessed;
}

DEFINE_ART_MODULE(novaddt::ChannelMapMaker)
