////////////////////////////////////////////////////////////////////////
// Class:       EvTimeFilter
// Module Type: filter
// File:        EvTimeFilter_module.cc
//
// Generated at Fri Mar 24 21:33:17 2017 by Andrey Sheshukov using artmod
// from cetpkgsupport v1_10_02.
////////////////////////////////////////////////////////////////////////

#include "art/Framework/Core/EDFilter.h"
#include "art/Framework/Core/ModuleMacros.h"
#include "art/Framework/Principal/Event.h"
#include "art/Framework/Principal/Handle.h"
#include "art/Framework/Principal/Run.h"
#include "art/Framework/Principal/SubRun.h"
#include "art/Utilities/InputTag.h"
#include "fhiclcpp/ParameterSet.h"
#include "messagefacility/MessageLogger/MessageLogger.h"

#include "DDTBaseDataProducts/EventHeader.h"	
#include "DDTBaseDataProducts/HitList.h"
#include <memory>

class EvTimeFilter;

class EvTimeFilter : public art::EDFilter {
public:
  explicit EvTimeFilter(fhicl::ParameterSet const & p);
  // The destructor generated by the compiler is fine for classes
  // without bare pointers or other resource use.

  // Plugins should not be copied or assigned.
  EvTimeFilter(EvTimeFilter const &) = delete;
  EvTimeFilter(EvTimeFilter &&) = delete;
  EvTimeFilter & operator = (EvTimeFilter const &) = delete;
  EvTimeFilter & operator = (EvTimeFilter &&) = delete;

  // Required functions.
  bool filter(art::Event & e) override;


private:

  // Declare member data here.

  art::InputTag fHeaderTag;
  art::InputTag fHitsTag;
};


EvTimeFilter::EvTimeFilter(fhicl::ParameterSet const & p):
    fHeaderTag   (p.get< std::string >("header_tag"  )),
    fHitsTag(p.get<std::string>("hits_tag"))
{ }

bool EvTimeFilter::filter(art::Event & e)
{
    const double msec=64000;
    auto header=e.getValidHandle<novaddt::EventHeader>(fHeaderTag);
    double hdr_time = header->timeStart/msec;

    auto Hits  =e.getValidHandle<novaddt::HitList>(fHitsTag);
    auto hits_size =  Hits->size();
    double hits_time_0 = hits_size?Hits->front().TDC().val/msec-hdr_time:0;
    double hits_time_1 = hits_size?Hits->back().TDC().val/msec-hdr_time:0;
    std::cout.setf(std::ios::fixed);
    std::cout.precision(2);

    std::cout
        <<header->eventNumber<<"\t "
        <<header->missingData<<"\t "
        <<header->datablockCount<<"\t "
        <<header->timeStart<<"\t "
        <<hdr_time<<"\t "<<hits_size<<"\t "
        <<hits_time_0<<"\t "
        <<hits_time_1<<std::endl;
    /// Check that the hits time span is not less than 4.9 ms
    return (hits_time_0+4.9<hits_time_1);
}

DEFINE_ART_MODULE(EvTimeFilter)
