////////////////////////////////////////////////////////////////////////
// Class:       sliceAna
// Module Type: analyzer
// File:        sliceAna_module.cc
//
// Generated at Sun Oct 30 01:06:14 2016 by Andrew S. Hirsch using artmod
// from cetpkgsupport v1_10_02.
////////////////////////////////////////////////////////////////////////

#include "art/Framework/Core/EDAnalyzer.h"
#include "art/Framework/Core/ModuleMacros.h"
#include "art/Framework/Principal/Event.h"
#include "art/Framework/Principal/Handle.h"
#include "art/Framework/Principal/Run.h"
#include "art/Framework/Principal/SubRun.h"
#include "art/Utilities/InputTag.h"
#include "fhiclcpp/ParameterSet.h"
#include "messagefacility/MessageLogger/MessageLogger.h"

#include "DDTBaseDataProducts/HitList.h"
#include "DDTBaseDataProducts/GroupedHitList.h"
#include "SuperNovaDDT/SliceSelector.h"
#include <memory>

#include "art/Framework/Services/Optional/TFileService.h"
#include "TTree.h"

namespace supernova {
  class sliceAna;
}

class supernova::sliceAna : public art::EDAnalyzer {
public:
  explicit sliceAna(fhicl::ParameterSet const & p);
  // The destructor generated by the compiler is fine for classes
  // without bare pointers or other resource use.

  // Plugins should not be copied or assigned.
  sliceAna(sliceAna const &) = delete;
  sliceAna(sliceAna &&) = delete;
  sliceAna & operator = (sliceAna const &) = delete;
  sliceAna & operator = (sliceAna &&) = delete;

  // Required functions.
  void analyze(art::Event const & e) override;


private:

  // Declare member data here.

  std::string fSlicesTag;
  novaddt::SliceSelector fSelector;

  TTree* fTree = 0;
  clusterInfo fClu;
  uint64_t    fEventNum = 0;
  std::vector<uint64_t> fTime;
  std::vector<uint64_t> fADC;
  std::vector<uint16_t> fX,fY,fZ;
  std::vector<uint16_t> fNx,fNy;

  void clearVector();
  void addToVector(clusterInfo&);
};

void supernova::sliceAna::clearVector(){
    fTime.clear();
    fADC.clear();
    fX.clear();
    fY.clear();
    fZ.clear();
    fNx.clear();
    fNy.clear();
}

void supernova::sliceAna::addToVector(clusterInfo& c){
    fTime.push_back(c.t);
    fADC.push_back(c.ADC());
    fX.push_back(c.crd[0]);
    fY.push_back(c.crd[1]);
    fZ.push_back(c.crd[2]);
    fNx.push_back(c.Nx());
    fNy.push_back(c.Ny());
}

supernova::sliceAna::sliceAna(fhicl::ParameterSet const & p)
  :
  EDAnalyzer(p),
  fSlicesTag   (p.get< std::string >("slices_tag"  )),
  fSelector    (p.get<fhicl::ParameterSet>("slice_cuts"))
{
    art::ServiceHandle<art::TFileService> tfs;
    fTree=tfs->make<TTree>("Slices","Slices analysis tree");
    fTree->Branch("event",&fEventNum,"num/l");
    fTree->Branch("tdc",&fTime);
    fTree->Branch("adc",&fADC);
    fTree->Branch("nx",&fNx);
    fTree->Branch("ny",&fNy);
    fTree->Branch("x",&fX);
    fTree->Branch("y",&fY);
    fTree->Branch("z",&fZ);

}

void supernova::sliceAna::analyze(art::Event const & e)
{
    fEventNum++;
    auto slices=e.getValidHandle<std::vector<novaddt::HitList>>(fSlicesTag);
    clearVector();
    for(const auto& slice: *slices){
        auto c=CalcCluster(slice);
        if(fSelector.checkCluster(c)) addToVector(c);
    }
    fTree->Fill();
}

DEFINE_ART_MODULE(supernova::sliceAna)
